use super::types::{Header, PSXImageConverter};
use tool_helper::{Error, Output};

pub struct RgbImage {
    image: image::RgbImage
}

impl RgbImage {
    pub fn new(image: image::RgbImage) -> RgbImage {
        RgbImage{image}
    }
}

impl PSXImageConverter for RgbImage {
    fn width(&self) -> u32 {
        image::RgbImage::width(&self.image)
    }

    fn height(&self) -> u32 {
        image::RgbImage::height(&self.image)
    }
}

pub struct RgbaImage {
    image: image::RgbaImage   
}

impl RgbaImage {
    pub fn new(image: image::RgbaImage) -> RgbaImage {
        RgbaImage{image}
    }
}

impl PSXImageConverter for RgbaImage {
    fn width(&self) -> u32 {
        image::RgbaImage::width(&self.image)
    }

    fn height(&self) -> u32 {
        image::RgbaImage::height(&self.image)
    }
}

pub fn encode<T: PSXImageConverter>(image: T, _: Output) -> Result<(), Error> {
    let width  = image.width();
    let height = image.height();

    if !Header::can_encode(width, height) {
        return Err(Error::from_text(format!("Image size {}, {} needs to be even", width, height)));
    }

    Ok(())
}